@@ -1,3 +1,5 @@
+from gettext import gettext as _
+
 import hashlib
 import os
 import re
@@ -7,20 +9,27 @@ from url_normalize import url_normalize
 from urllib.parse import urlparse
 
 from django.db import models
-from django.conf import settings
-from django.core.files.storage import FileSystemStorage
+from django.contrib.auth.models import Group
 from django.contrib.postgres import fields
 from django.shortcuts import redirect
 
+from django_currentuser.middleware import get_current_authenticated_user
+from django_lifecycle import hook
+
+from guardian.models.models import GroupObjectPermission, UserObjectPermission
+from guardian.shortcuts import assign_perm
+
 from pulpcore.plugin.download import DownloaderFactory
 from pulpcore.plugin.models import (
-    Artifact,
+    AutoAddObjPermsMixin,
+    AutoDeleteObjPermsMixin,
+    BaseModel,
     Content,
     ContentGuard,
-    BaseModel,
     Remote,
     Repository,
     RepositoryVersionDistribution,
+    Upload as CoreUpload,
 )
 from pulpcore.plugin.repo_version_utils import remove_duplicates, validate_repo_version
 
@@ -186,7 +195,75 @@ class Tag(Content):
         unique_together = (("name", "tagged_manifest"),)
 
 
-class ContainerRepository(Repository):
+class ContainerNamespace(BaseModel, AutoAddObjPermsMixin):
+    """
+    Namespace for the container registry.
+    """
+
+    name = models.CharField(max_length=255, db_index=True)
+    ACCESS_POLICY_VIEWSET_NAME = "pulp_container/namespaces"
+
+    def create_namespace_group(self, permissions, parameters):
+        """
+        Creates a namespace group and optionally adds the current user to it.
+
+        The parameters are specified as a dictionary with the following keys:
+
+        "group_type" - the type of group - owners, collaborators, or consumers
+        "add_user_to_group" - a boolean that specifies if the current user should be added to the
+                              group.
+
+        The permissions are object level permissions assigned to the group.
+        """
+
+        group_type = parameters["group_type"]
+        add_user_to_group = parameters["add_user_to_group"]
+        group = Group.objects.create(
+            name="{}.{}.{}".format("container.namespace", group_type, self.name)
+        )
+        current_user = get_current_authenticated_user()
+        owners_group = Group.objects.get(
+            name="{}.{}".format("container.namespace.owners", self.name)
+        )
+        assign_perm("auth.change_group", owners_group, group)
+        assign_perm("auth.view_group", owners_group, group)
+        if add_user_to_group:
+            current_user.groups.add(group)
+        self.add_for_groups(permissions, group.name)
+
+    @hook("before_delete")
+    def delete_groups_and_user_obj_perms(self):
+        """
+        Delete all auto created groups associated with this Namespace and user object
+        permissions.
+        """
+        group_name_regex = r"container.namespace.(.*).{}".format(self.name)
+        Group.objects.filter(name__regex=group_name_regex).delete()
+        UserObjectPermission.objects.filter(object_pk=self.pk).delete()
+        GroupObjectPermission.objects.filter(object_pk=self.pk).delete()
+
+    class Meta:
+        unique_together = (("name",),)
+        permissions = [
+            ("namespace_add_containerdistribution", "Add any distribution to a namespace"),
+            ("namespace_delete_containerdistribution", "Delete any distribution from a namespace"),
+            ("namespace_view_containerdistribution", "View any distribution in a namespace"),
+            ("namespace_pull_containerdistribution", "Pull from any distribution in a namespace"),
+            ("namespace_push_containerdistribution", "Push to any distribution in a namespace"),
+            ("namespace_change_containerdistribution", "Change any distribution in a namespace"),
+            ("namespace_view_containerpushrepository", "View any push repository in a namespace"),
+            (
+                "namespace_modify_content_containerpushrepository",
+                "Modify content in any push repository in a namespace",
+            ),
+        ]
+
+
+class ContainerRepository(
+    Repository,
+    AutoAddObjPermsMixin,
+    AutoDeleteObjPermsMixin,
+):
     """
     Repository for "container" content.
 
@@ -197,9 +274,16 @@ class ContainerRepository(Repository):
     TYPE = "container"
     CONTENT_TYPES = [Blob, Manifest, Tag]
     PUSH_ENABLED = False
+    ACCESS_POLICY_VIEWSET_NAME = "repositories/container/container"
 
     class Meta:
         default_related_name = "%(app_label)s_%(model_name)s"
+        permissions = [
+            ("sync_containerrepository", "Can start a sync task"),
+            ("modify_content_containerrepository", "Can modify content in a repository"),
+            ("build_image_containerrepository", "Can use the image builder in a repository"),
+            ("delete_containerrepository_versions", "Can delete repository versions"),
+        ]
 
     def finalize_new_version(self, new_version):
         """
@@ -212,12 +296,12 @@ class ContainerRepository(Repository):
         validate_repo_version(new_version)
 
 
-class ContainerPushRepository(Repository):
+class ContainerPushRepository(Repository, AutoAddObjPermsMixin, AutoDeleteObjPermsMixin):
     """
     Repository for "container" content.
 
     This repository type is designed for the read and write registry usecase. It will be
-    automatically instanciated on authorised push to nonexisting repositories.
+    automatically instantiated on authorised push to nonexisting repositories.
     With this repository type, all but the latest repository_version are solely of historical
     interest.
     """
@@ -225,9 +309,41 @@ class ContainerPushRepository(Repository):
     TYPE = "container-push"
     CONTENT_TYPES = [Blob, Manifest, Tag]
     PUSH_ENABLED = True
+    ACCESS_POLICY_VIEWSET_NAME = "repositories/container/container-push"
+
+    def add_perms_to_distribution_group(self, permissions, parameters):
+        """
+        Adds push repository object permissions to a distribution group.
+
+        The parameters are specified as a dictionary with the following keys:
+
+        "group_type" - the type of group - owners, collaborators, or consumers
+        "add_user_to_group" - a boolean that specifies if the current user should be added to the
+                              group.
+
+        The permissions are object level permissions assigned to the group.
+        """
+
+        group_type = parameters["group_type"]
+        add_user_to_group = parameters["add_user_to_group"]
+        try:
+            suffix = ContainerDistribution.objects.get(repository=self).pk
+        except ContainerDistribution.DoesNotExist:
+            # The distribution has not been created yet
+            return
+        group = Group.objects.get(
+            name="{}.{}.{}".format("container.distribution", group_type, suffix)
+        )
+        current_user = get_current_authenticated_user()
+        if add_user_to_group:
+            current_user.groups.add(group)
+        self.add_for_groups(permissions, group.name)
 
     class Meta:
         default_related_name = "%(app_label)s_%(model_name)s"
+        permissions = [
+            ("modify_content_containerpushrepository", "Can modify content in a push repository")
+        ]
 
     def finalize_new_version(self, new_version):
         """
@@ -240,7 +356,7 @@ class ContainerPushRepository(Repository):
         validate_repo_version(new_version)
 
 
-class ContainerRemote(Remote):
+class ContainerRemote(Remote, AutoAddObjPermsMixin, AutoDeleteObjPermsMixin):
     """
     A Remote for ContainerContent.
 
@@ -256,6 +372,7 @@ class ContainerRemote(Remote):
     exclude_tags = fields.ArrayField(models.CharField(max_length=255, null=True), null=True)
 
     TYPE = "container"
+    ACCESS_POLICY_VIEWSET_NAME = "remotes/container/container"
 
     @property
     def download_factory(self):
@@ -328,12 +445,28 @@ class ContainerRemote(Remote):
         default_related_name = "%(app_label)s_%(model_name)s"
 
 
-class ContainerDistribution(RepositoryVersionDistribution):
+class ContainerDistribution(RepositoryVersionDistribution, AutoAddObjPermsMixin):
     """
-    A container distribution defines how a publication is distributed by Pulp's webserver.
+    A container distribution defines how a repository version is distributed by Pulp's webserver.
     """
 
     TYPE = "container"
+    ACCESS_POLICY_VIEWSET_NAME = "distributions/container/container"
+
+    namespace = models.ForeignKey(
+        ContainerNamespace,
+        on_delete=models.CASCADE,
+        related_name="container_distributions",
+        null=True,
+    )
+    private = models.BooleanField(
+        default=False,
+        help_text=_(
+            "Restrict pull access to explicitly authorized users. "
+            "Defaults to unrestricted pull access."
+        ),
+    )
+    description = models.TextField(null=True)
 
     def get_repository_version(self):
         """
@@ -354,8 +487,71 @@ class ContainerDistribution(RepositoryVersionDistribution):
             url = self.content_guard.cast().preauthenticate_url(url)
         return redirect(url)
 
+    def create_distribution_group(self, permissions, parameters):
+        """
+        Creates a distribution group and optionally adds the current user to it.
+
+        The parameters are specified as a dictionary with the following keys:
+
+        "group_type" - the type of group - owners, collaborators, or consumers
+        "add_user_to_group" - a boolean that specifies if the current user should be added to the
+                              group.the "model_field" for the instance.
+
+        The permissions are object level permissions assigned to the group.
+        """
+
+        group_type = parameters["group_type"]
+        add_user_to_group = parameters["add_user_to_group"]
+
+        group = Group.objects.create(
+            name="{}.{}.{}".format("container.distribution", group_type, self.pk)
+        )
+        current_user = get_current_authenticated_user()
+        owners_group = Group.objects.get(
+            name="{}.{}".format("container.distribution.owners", self.pk)
+        )
+        assign_perm("auth.change_group", owners_group, group)
+        assign_perm("auth.view_group", owners_group, group)
+        if add_user_to_group:
+            current_user.groups.add(group)
+        self.add_for_groups(permissions, group.name)
+
+    def add_push_repository_perms_to_distribution_group(self, permissions, parameters):
+        """
+        Adds permissions related to ContainerPushRepository to a distribution group.
+
+        The parameters are specified as a dictionary with the following keys:
+
+        "group_type" - the type of group - owners, collaborators, or consumers
+
+        The permissions are ContainerPushRepository object level permissions assigned to the
+        group. The repository is the one that is associated with the ContainerDistribution.
+        """
+
+        group_type = parameters["group_type"]
+        group = Group.objects.get(
+            name="{}.{}.{}".format("container.distribution", group_type, self.pk)
+        )
+        if isinstance(self.repository, ContainerPushRepository):
+            self.repository.add_for_groups(permissions, group.name)
+
+    @hook("before_delete")
+    def delete_groups_and_user_obj_perms(self):
+        """
+        Delete all auto created groups associated with this Distribution and user object
+        permissions.
+        """
+        group_name_regex = r"container.distribution.(.*).{}".format(self.pk)
+        Group.objects.filter(name__regex=group_name_regex).delete()
+        UserObjectPermission.objects.filter(object_pk=self.pk).delete()
+        GroupObjectPermission.objects.filter(object_pk=self.pk).delete()
+
     class Meta:
         default_related_name = "%(app_label)s_%(model_name)s"
+        permissions = [
+            ("pull_containerdistribution", "Can pull from a registry repo"),
+            ("push_containerdistribution", "Can push into the registry repo"),
+        ]
 
 
 INCOMPLETE_EXT = ".part"
@@ -367,60 +563,13 @@ def generate_filename(instance, filename):
     return time.strftime(filename)
 
 
-class Upload(BaseModel):
+class Upload(CoreUpload):
     """
     Model for tracking Blob uploads.
     """
 
     repository = models.ForeignKey(Repository, related_name="uploads", on_delete=models.CASCADE)
 
-    offset = models.BigIntegerField(default=0)
-
-    file = models.FileField(
-        max_length=255,
-        null=True,
-        upload_to=generate_filename,
-        storage=FileSystemStorage(location=settings.CHUNKED_UPLOAD_DIR),
-    )
-
-    upload_dir = os.path.join(settings.CHUNKED_UPLOAD_DIR, "container")
-
-    size = models.IntegerField(null=True)
-    md5 = models.CharField(max_length=32, null=True)
-    sha1 = models.CharField(max_length=40, null=True)
-    sha224 = models.CharField(max_length=56, null=True)
-    sha256 = models.CharField(max_length=64, null=True)
-    sha384 = models.CharField(max_length=96, null=True)
-    sha512 = models.CharField(max_length=128, null=True)
-
-    def append_chunk(self, chunk, chunk_size=None, save=True):
-        """Method for appending a chunk to a file."""
-        hashers = {}
-        for algorithm in Artifact.DIGEST_FIELDS:
-            hashers[algorithm] = getattr(hashlib, algorithm)()
-
-        self.file.close()
-        self.file.open(mode="ab")  # mode = append+binary
-        while True:
-            subchunk = chunk.read(2000000)
-            if not subchunk:
-                break
-            self.file.write(subchunk)
-            for algorithm in Artifact.DIGEST_FIELDS:
-                hashers[algorithm].update(subchunk)
-
-        if chunk_size is not None:
-            self.offset += chunk_size
-        elif hasattr(chunk, "size"):
-            self.offset += chunk.size
-        else:
-            self.offset = self.file.size
-        if save:
-            self.save()
-        self.file.close()  # Flush
-        for algorithm in Artifact.DIGEST_FIELDS:
-            setattr(self, algorithm, hashers[algorithm].hexdigest())
-
 
 def _gen_secret():
     return os.urandom(32)
