@@ -8,35 +8,55 @@ import json
 import logging
 import hashlib
 import re
+from collections import namedtuple
+import time
+
+from urllib.parse import urlparse, urlunparse, parse_qs, urlencode
 from tempfile import NamedTemporaryFile
 
 from django.core.files.storage import default_storage as storage
-
+from django.core.files.base import ContentFile, File
 from django.db import IntegrityError, transaction
 from django.shortcuts import get_object_or_404
 
 from django.conf import settings
-from django.core.files.base import ContentFile
 
-from pulpcore.plugin.models import Artifact, ContentArtifact
+from guardian.shortcuts import get_objects_for_user
+
+from pulpcore.plugin.models import Artifact, ContentArtifact, Task, UploadChunk
+from pulpcore.plugin.files import PulpTemporaryUploadedFile
+from pulpcore.plugin.tasking import add_and_remove, enqueue_with_reservation
 from rest_framework.exceptions import (
     AuthenticationFailed,
     NotAuthenticated,
     PermissionDenied,
     NotFound,
     ParseError,
+    Throttled,
     ValidationError,
 )
+from rest_framework.generics import ListAPIView
+from rest_framework.pagination import BasePagination
+from rest_framework.permissions import IsAuthenticated
 from rest_framework.renderers import BaseRenderer, JSONRenderer
 from rest_framework.response import Response
+from rest_framework.serializers import ModelSerializer
+from rest_framework.settings import api_settings
 from rest_framework.viewsets import ViewSet
 from rest_framework.views import APIView
 
 from pulp_container.app import models, serializers
+from pulp_container.app.access_policy import RegistryAccessPolicy
 from pulp_container.app.authorization import AuthorizationService
 from pulp_container.app.redirects import FileStorageRedirects, S3StorageRedirects
-from pulp_container.app.token_verification import TokenAuthentication, TokenPermission
+from pulp_container.app.token_verification import (
+    RegistryAuthentication,
+    TokenAuthentication,
+    RegistryPermission,
+    TokenPermission,
+)
 
+FakeView = namedtuple("FakeView", ["action", "get_object"])
 
 log = logging.getLogger(__name__)
 
@@ -140,8 +160,8 @@ class UploadResponse(Response):
         """
         headers = {
             "Docker-Upload-UUID": upload.pk,
-            "Location": "/v2/{path}/blobs/uploads/{pk}".format(path=path, pk=upload.pk),
-            "Range": "0-{offset}".format(offset=upload.file.size),
+            "Location": f"/v2/{path}/blobs/uploads/{upload.pk}",
+            "Range": "0-{offset}".format(offset=int(upload.size - 1)),
             "Content-Length": content_length,
         }
         super().__init__(headers=headers, status=202)
@@ -189,7 +209,6 @@ class BlobResponse(Response):
         artifact = blob._artifacts.get()
         size = artifact.size
 
-        log.info("digest: {digest}".format(digest=blob.digest))
         headers = {
             "Docker-Content-Digest": blob.digest,
             "Location": "/v2/{path}/blobs/{digest}".format(path=path, digest=blob.digest),
@@ -211,9 +230,26 @@ class ContainerRegistryApiMixin:
     It sets token authentication and token permission.
     """
 
-    authentication_classes = [TokenAuthentication]
-    permission_classes = [TokenPermission]
     schema = None
+    TOKEN_ERROR_CODES = ("invalid_token", "insufficient_scope")
+
+    @property
+    def authentication_classes(self):
+        """
+        List of authentication classes to check for this view.
+        """
+        if settings.get("TOKEN_AUTH_DISABLED", False):
+            return [RegistryAuthentication]
+        return [TokenAuthentication]
+
+    @property
+    def permission_classes(self):
+        """
+        List of permission classes to check for this view.
+        """
+        if settings.get("TOKEN_AUTH_DISABLED", False):
+            return [RegistryPermission]
+        return [TokenPermission]
 
     @property
     def default_response_headers(self):
@@ -246,7 +282,15 @@ class ContainerRegistryApiMixin:
             else:
                 code = "UNSUPPORTED"
             exc.detail = {"errors": [{"code": code, "message": detail, "detail": {}}]}
-        return super().handle_exception(exc)
+
+        response = super().handle_exception(exc)
+
+        # the auth header is available when the response object is initialized
+        error_code = getattr(detail, "code", "")
+        if error_code in self.TOKEN_ERROR_CODES:
+            response["Www-Authenticate"] += f',error="{error_code}"'
+
+        return response
 
     def get_drv_pull(self, path):
         """
@@ -292,6 +336,14 @@ class ContainerRegistryApiMixin:
                         distribution = dist_serializer.create(dist_serializer.validated_data)
                 except ValidationError:
                     raise RepositoryInvalid(name=path)
+                except IntegrityError:
+                    # Seems like another process created our stuff already. Retry fetching it.
+                    distribution = models.ContainerDistribution.objects.get(base_path=path)
+                    repository = distribution.repository
+                    if repository:
+                        repository = repository.cast()
+                        if not repository.PUSH_ENABLED:
+                            raise RepositoryInvalid(name=path, message="Repository is read-only.")
             else:
                 raise RepositoryNotFound(name=path)
         else:
@@ -300,6 +352,26 @@ class ContainerRegistryApiMixin:
                 repository = repository.cast()
                 if not repository.PUSH_ENABLED:
                     raise RepositoryInvalid(name=path, message="Repository is read-only.")
+            elif create:
+                try:
+                    with transaction.atomic():
+                        repo_serializer = serializers.ContainerPushRepositorySerializer(
+                            data={"name": path}, context={"request": request}
+                        )
+                        repo_serializer.is_valid(raise_exception=True)
+                        repository = repo_serializer.create(repo_serializer.validated_data)
+                        distribution.repository = repository
+                        distribution.save()
+                except IntegrityError:
+                    # Seems like another process created our stuff already. Retry fetching it.
+                    distribution = models.ContainerDistribution.objects.get(pk=distribution.pk)
+                    repository = distribution.repository
+                    if repository:
+                        repository = repository.cast()
+                        if not repository.PUSH_ENABLED:
+                            raise RepositoryInvalid(name=path, message="Repository is read-only.")
+                    else:
+                        raise RepositoryNotFound(name=path)
             else:
                 raise RepositoryNotFound(name=path)
         return distribution, repository
@@ -313,25 +385,16 @@ class BearerTokenView(APIView):
     # Allow everyone to access but still value authenticated users.
     permission_classes = []
 
-    ANONYMOUS_USER = ""
-    EMPTY_ACCESS_SCOPE = "::"
-
     def get(self, request):
         """Handles GET requests for the /token/ endpoint."""
-        account = request.query_params.get("account", self.ANONYMOUS_USER)
         try:
             service = request.query_params["service"]
         except KeyError:
             raise ParseError(detail="No service name provided.")
-        scope = request.query_params.get("scope", self.EMPTY_ACCESS_SCOPE)
+        scope = request.query_params.get("scope", "")
 
-        if account != self.ANONYMOUS_USER:
-            if not request.user.is_authenticated:
-                raise ParseError(detail="Authentication failed.")
-            if account != request.user.username:
-                raise ParseError(detail="Username mismatch.")
-
-        data = AuthorizationService().generate_token(account, service, scope)
+        authorization_service = AuthorizationService(self.request.user, service, scope)
+        data = authorization_service.generate_token()
         return Response(data=data)
 
 
@@ -340,41 +403,175 @@ class VersionView(ContainerRegistryApiMixin, APIView):
     Handles requests to the /v2/ endpoint.
     """
 
+    @property
+    def permission_classes(self):
+        """
+        List of permission classes to check for this view.
+        """
+        if settings.get("TOKEN_AUTH_DISABLED", False):
+            return [IsAuthenticated]
+        return [TokenPermission]
+
     def get(self, request):
         """Handles GET requests for the /v2/ endpoint."""
         return Response(data={})
 
 
-class CatalogView(ContainerRegistryApiMixin, APIView):
+class ContainerCatalogSerializer(ModelSerializer):
+    """
+    Serializer for Distributions in the _catalog endpoint of the registry.
+    """
+
+    class Meta:
+        model = models.ContainerDistribution
+        fields = ["base_path"]
+
+
+class ContainerCatalogPagination(BasePagination):
+    """
+    Pagination class to paginate repositories by names according to the registry api specification.
+    """
+
+    def paginate_queryset(self, queryset, request, view=None):
+        """
+        Analyse the pagination parameters and prepare the queryset.
+        """
+        try:
+            self.n = int(request.query_params.get("n"))
+        except Exception:
+            self.n = api_settings.PAGE_SIZE
+        else:
+            if self.n > 10 * api_settings.PAGE_SIZE:
+                self.n = 10 * api_settings.PAGE_SIZE
+            if self.n < 0:
+                self.n = 0
+        last = request.query_params.get("last")
+        self.url = request.build_absolute_uri()
+
+        if last:
+            queryset = queryset.filter(base_path__gt=last)
+        return queryset.order_by("base_path")[: self.n]
+
+    def get_paginated_response(self, data):
+        """
+        Prepare the paginated container _catalog response.
+        """
+        headers = {}
+        repositories_names = [repo["base_path"] for repo in data]
+        if self.n and len(repositories_names) == self.n:
+            # There's a high chance we haven't gotten all entries here.
+            parsed_url = list(urlparse(self.url))
+            query_params = parse_qs(parsed_url[4])
+            query_params["n"] = str(self.n)
+            query_params["last"] = repositories_names[-1]
+            parsed_url[4] = urlencode(query_params)
+            url = urlunparse(parsed_url)
+            headers["Link"] = f'<{url}>; rel="next"'
+        return Response(headers=headers, data={"repositories": repositories_names})
+
+
+class CatalogView(ContainerRegistryApiMixin, ListAPIView):
     """
     Handles requests to the /v2/_catalog endpoint
     """
 
-    def get(self, request):
-        """Handles GET requests for the /v2/_catalog endpoint."""
-        repositories_names = models.ContainerDistribution.objects.values_list(
-            "base_path", flat=True
+    queryset = models.ContainerDistribution.objects.all().only("base_path")
+    serializer_class = ContainerCatalogSerializer
+    pagination_class = ContainerCatalogPagination
+    access_policy_class = RegistryAccessPolicy()
+
+    def get_queryset(self, *args, **kwargs):
+        """Filter the queryset based on public repositories and assigned permissions."""
+        queryset = super().get_queryset()
+
+        distribution_permission = "container.pull_containerdistribution"
+        namespace_permission = "container.namespace_pull_containerdistribution"
+
+        public_repositories = queryset.filter(private=False)
+        repositories_by_distribution = get_objects_for_user(
+            self.request.user, distribution_permission, queryset
         )
-        return Response(data={"repositories": list(repositories_names)})
+
+        namespace_refs = queryset.values_list("namespace", flat=True)
+        namespaces = models.ContainerNamespace.objects.filter(pk__in=namespace_refs)
+        repositories_by_namespace = get_objects_for_user(
+            self.request.user, namespace_permission, namespaces
+        )
+        repositories_by_namespace = queryset.filter(namespace__in=repositories_by_namespace)
+
+        accessible_repositories = repositories_by_distribution & repositories_by_namespace
+        return (public_repositories | accessible_repositories).distinct()
 
 
-class TagsListView(ContainerRegistryApiMixin, APIView):
+class ContainerTagListSerializer(ModelSerializer):
+    """
+    Serializer for Tags in the tags list endpoint of the registry.
+    """
+
+    class Meta:
+        model = models.Tag
+        fields = ["name"]
+
+
+class ContainerTagListPagination(BasePagination):
+    """
+    Pagination class to paginate tags by names according to the registry api specification.
+    """
+
+    def paginate_queryset(self, queryset, request, view=None):
+        """
+        Analyse the pagination parameters and prepare the queryset.
+        """
+        try:
+            self.n = int(request.query_params.get("n"))
+        except Exception:
+            self.n = api_settings.PAGE_SIZE
+        else:
+            if self.n > 10 * api_settings.PAGE_SIZE:
+                self.n = 10 * api_settings.PAGE_SIZE
+            if self.n < 0:
+                self.n = 0
+        last = request.query_params.get("last")
+        self.url = request.build_absolute_uri()
+        self.path = request.resolver_match.kwargs["path"]
+
+        if last:
+            queryset = queryset.filter(name__gt=last)
+        return queryset.order_by("name")[: self.n]
+
+    def get_paginated_response(self, data):
+        """
+        Prepare the paginated container _catalog response.
+        """
+        headers = {}
+        tag_names = [tag["name"] for tag in data]
+        if self.n and len(tag_names) == self.n:
+            # There's a high chance we haven't gotten all entries here.
+            parsed_url = list(urlparse(self.url))
+            query_params = parse_qs(parsed_url[4])
+            query_params["n"] = str(self.n)
+            query_params["last"] = tag_names[-1]
+            parsed_url[4] = urlencode(query_params)
+            url = urlunparse(parsed_url)
+            headers["Link"] = f'<{url}>; rel="next"'
+        return Response(headers=headers, data={"name": self.path, "tags": tag_names})
+
+
+class TagsListView(ContainerRegistryApiMixin, ListAPIView):
     """
     Handles requests to the /v2/<repo>/tags/list endpoint
     """
 
-    def get(self, request, path):
+    serializer_class = ContainerTagListSerializer
+    pagination_class = ContainerTagListPagination
+
+    def get_queryset(self):
         """
         Handles GET requests to the /v2/<repo>/tags/list endpoint
         """
+        path = self.request.resolver_match.kwargs["path"]
         _, _, repository_version = self.get_drv_pull(path)
-        tags = {"name": path, "tags": set()}
-        for c in repository_version.content:
-            c = c.cast()
-            if isinstance(c, models.Tag):
-                tags["tags"].add(c.name)
-        tags["tags"] = list(tags["tags"])
-        return Response(data=tags)
+        return models.Tag.objects.filter(pk__in=repository_version.content).only("name")
 
 
 class BlobUploads(ContainerRegistryApiMixin, ViewSet):
@@ -389,12 +586,11 @@ class BlobUploads(ContainerRegistryApiMixin, ViewSet):
 
     def create(self, request, path):
         """
-        This methods handles the creation of an upload.
+        Create a new upload.
         """
         _, repository = self.get_dr_push(request, path, create=True)
 
-        upload = models.Upload(repository=repository)
-        upload.file.save(name="", content=ContentFile(""), save=False)
+        upload = models.Upload(repository=repository, size=0)
         upload.save()
         response = UploadResponse(upload=upload, path=path, content_length=0, request=request)
 
@@ -402,7 +598,7 @@ class BlobUploads(ContainerRegistryApiMixin, ViewSet):
 
     def partial_update(self, request, path, pk=None):
         """
-        This methods handles uploading of a chunk to an existing upload.
+        Process a chunk that will be appended to an existing upload.
         """
         _, repository = self.get_dr_push(request, path)
         chunk = request.META["wsgi.input"]
@@ -413,47 +609,72 @@ class BlobUploads(ContainerRegistryApiMixin, ViewSet):
 
         if whole:
             start = 0
-            end = chunk.size - 1
         else:
             content_range = request.META.get("HTTP_CONTENT_RANGE", "")
             match = self.content_range_pattern.match(content_range)
-            if not match:
-                start = 0
-                end = 0
-                chunk_size = 0
-            else:
-                start = int(match.group("start"))
-                end = int(match.group("end"))
-                chunk_size = end - start + 1
+            start = 0 if not match else int(match.group("start"))
 
         upload = get_object_or_404(models.Upload, repository=repository, pk=pk)
 
-        if upload.offset != start:
-            raise Exception
-        upload.append_chunk(chunk, chunk_size=chunk_size)
-        upload.save()
-        return UploadResponse(
-            upload=upload, path=path, content_length=upload.file.size, request=request
-        )
+        chunk = ContentFile(chunk.read())
+        with transaction.atomic():
+            if upload.size != start:
+                raise Exception
+
+            upload.append(chunk, upload.size)
+            upload.size += chunk.size
+            upload.save()
+
+        return UploadResponse(upload=upload, path=path, content_length=chunk.size, request=request)
 
     def put(self, request, path, pk=None):
-        """Handles creation of Uploads."""
+        """
+        Create a blob from uploaded chunks.
+        """
         _, repository = self.get_dr_push(request, path)
 
         digest = request.query_params["digest"]
-        upload = models.Upload.objects.get(pk=pk, repository=repository)
-
-        if upload.sha256 == digest[len("sha256:") :]:
+        # Try to see if the client came back after we told it to backoff with the ``Throttled``
+        # exception. In that case we answer based on the task state, or make it backoff again.
+        # This mechanism seems to work with podman but not with docker. However we let the task run
+        # anyway, since all clients will look with a HEAD request before attemting to upload a blob
+        # again.
+        try:
+            upload = models.Upload.objects.get(pk=pk, repository=repository)
+        except models.Upload.DoesNotExist as e_upload:
+            # Upload has been deleted => task has started or even finished
             try:
-                artifact = Artifact(
-                    file=upload.file.name,
-                    md5=upload.md5,
-                    sha1=upload.sha1,
-                    sha256=upload.sha256,
-                    sha384=upload.sha384,
-                    sha512=upload.sha512,
-                    size=upload.file.size,
-                )
+                task = Task.objects.filter(
+                    name__endswith="add_and_remove",
+                    reserved_resources_record__resource=f"upload:{pk}",
+                ).last()
+            except Task.DoesNotExist:
+                # No upload and no task for it => the upload probably never existed
+                # return 404
+                raise e_upload
+
+            if task.state == "completed":
+                task.delete()
+                blob = models.Blob.objects.get(digest=digest)
+                return BlobResponse(blob, path, 201, request)
+            elif task.state in ["waiting", "running"]:
+                raise Throttled(wait=5)
+            else:
+                task.delete()
+                raise Exception("Failed.")
+
+        chunks = UploadChunk.objects.filter(upload=upload).order_by("offset")
+
+        with NamedTemporaryFile("ab") as temp_file:
+            for chunk in chunks:
+                temp_file.write(chunk.file.read())
+            temp_file.flush()
+
+            uploaded_file = PulpTemporaryUploadedFile.from_file(File(open(temp_file.name, "rb")))
+
+        if uploaded_file.hashers["sha256"].hexdigest() == digest[len("sha256:") :]:
+            try:
+                artifact = Artifact.init_and_validate(uploaded_file)
                 artifact.save()
             except IntegrityError:
                 artifact = Artifact.objects.get(sha256=artifact.sha256)
@@ -470,12 +691,31 @@ class BlobUploads(ContainerRegistryApiMixin, ViewSet):
             except IntegrityError:
                 pass
 
-            with repository.new_version() as new_version:
-                new_version.add_content(models.Blob.objects.filter(pk=blob.pk))
-
             upload.delete()
 
-            return BlobResponse(blob, path, 201, request)
+            job = enqueue_with_reservation(
+                add_and_remove,
+                [f"upload:{pk}", repository],
+                kwargs={
+                    "repository_pk": repository.pk,
+                    "add_content_units": [blob.pk],
+                    "remove_content_units": [],
+                },
+            )
+
+            # Wait a small amount of time
+            for dummy in range(3):
+                time.sleep(1)
+                task = Task.objects.get(pk=job.id)
+                if task.state == "completed":
+                    task.delete()
+                    return BlobResponse(blob, path, 201, request)
+                elif task.state in ["waiting", "running"]:
+                    continue
+                else:
+                    task.delete()
+                    raise Exception("Failed.")
+            raise Throttled(wait=5)
         else:
             raise Exception("The digest did not match")
 
@@ -600,14 +840,34 @@ class Manifests(RedirectsMixin, ContainerRegistryApiMixin, ViewSet):
         try:
             tag.save()
         except IntegrityError:
-            pass
-        with repository.new_version() as new_version:
-            new_version.add_content(models.Manifest.objects.filter(digest=manifest.digest))
-            new_version.remove_content(models.Tag.objects.filter(name=tag.name))
-            new_version.add_content(
-                models.Tag.objects.filter(name=tag.name, tagged_manifest=manifest)
-            )
-        return ManifestResponse(manifest, path, request, status=201)
+            tag = models.Tag.objects.get(name=tag.name, tagged_manifest=manifest)
+
+        tags_to_remove = models.Tag.objects.filter(
+            pk__in=repository.latest_version().content.all(), name=tag
+        ).exclude(tagged_manifest=manifest)
+        job = enqueue_with_reservation(
+            add_and_remove,
+            [repository],
+            kwargs={
+                "repository_pk": repository.pk,
+                "add_content_units": [tag.pk, manifest.pk],
+                "remove_content_units": tags_to_remove.values_list("pk"),
+            },
+        )
+
+        # Wait a small amount of time
+        for dummy in range(3):
+            time.sleep(1)
+            task = Task.objects.get(pk=job.id)
+            if task.state == "completed":
+                task.delete()
+                return ManifestResponse(manifest, path, request, status=201)
+            elif task.state in ["waiting", "running"]:
+                continue
+            else:
+                task.delete()
+                raise Exception("Failed.")
+        raise Throttled(wait=5)
 
     def receive_artifact(self, chunk):
         """Handles assembling of Manifest as it's being uploaded."""
