@@ -242,6 +242,9 @@ class ContainerDistributionSerializer(RepositoryVersionDistributionSerializer):
         view_name="pulp_container/namespaces-detail",
         help_text=_("Namespace this distribution belongs to."),
     )
+    description = serializers.CharField(
+        help_text=_("An optional description."), required=False, allow_null=True
+    )
 
     def validate(self, data):
         """
@@ -265,18 +268,20 @@ class ContainerDistributionSerializer(RepositoryVersionDistributionSerializer):
                         "repository version."
                     )
                 )
-        base_path = validated_data["base_path"]
-        namespace_name = base_path.split("/")[0]
 
-        if "namespace" in validated_data:
-            if validated_data["namespace"].name != namespace_name:
-                raise serializers.ValidationError(
-                    _("Selected namespace does not match first component of base_path.")
+        if "base_path" in validated_data:
+            base_path = validated_data["base_path"]
+            namespace_name = base_path.split("/")[0]
+
+            if "namespace" in validated_data:
+                if validated_data["namespace"].name != namespace_name:
+                    raise serializers.ValidationError(
+                        _("Selected namespace does not match first component of base_path.")
+                    )
+            else:
+                validated_data["namespace"] = ContainerNamespaceSerializer.get_or_create(
+                    {"name": namespace_name}
                 )
-        else:
-            validated_data["namespace"] = ContainerNamespaceSerializer.get_or_create(
-                {"name": namespace_name}
-            )
         return validated_data
 
     class Meta:
@@ -284,6 +289,8 @@ class ContainerDistributionSerializer(RepositoryVersionDistributionSerializer):
         fields = tuple(set(RepositoryVersionDistributionSerializer.Meta.fields) - {"base_url"}) + (
             "registry_path",
             "namespace",
+            "private",
+            "description",
         )
 
 
@@ -328,7 +335,11 @@ class TagOperationSerializer(serializers.Serializer):
         latest_version = new_data["repository"].latest_version()
         if not latest_version:
             raise serializers.ValidationError(
-                _("The latest repository version of '{}' was not found".format(data["repository"]))
+                _(
+                    "The latest repository version of '{}' was not found".format(
+                        new_data["repository"]
+                    )
+                )
             )
 
         new_data["latest_version"] = latest_version
@@ -498,6 +509,56 @@ class ManifestCopySerializer(CopySerializer):
     )
 
 
+class RemoveImageSerializer(serializers.Serializer):
+    """
+    A serializer for parsing and validating data associated with the image removal.
+    """
+
+    digest = serializers.CharField(help_text="sha256 of the Manifest file")
+
+    def validate(self, data):
+        """
+        Validate and extract the latest repository version, manifest, and tags from the passed data.
+        """
+        new_data = {}
+        new_data.update(self.initial_data)
+
+        latest_version = new_data["repository"].latest_version()
+        if not latest_version:
+            raise serializers.ValidationError(
+                _(
+                    "The latest repository version of '{}' was not found".format(
+                        new_data["repository"]
+                    )
+                )
+            )
+
+        new_data["latest_version"] = latest_version
+
+        try:
+            manifest = models.Manifest.objects.get(
+                pk__in=new_data["latest_version"].content.all(), digest=new_data["digest"]
+            )
+        except models.Manifest.DoesNotExist:
+            raise serializers.ValidationError(
+                _(
+                    "A manifest with the digest '{}' does not "
+                    "exist in the latest repository version '{}'".format(
+                        new_data["digest"], new_data["latest_version"]
+                    )
+                )
+            )
+
+        new_data["manifest"] = manifest
+
+        tags_pks = models.Tag.objects.filter(
+            pk__in=new_data["latest_version"].content.all(), tagged_manifest=new_data["manifest"]
+        ).values_list("pk", flat=True)
+        new_data["tags_pks"] = tags_pks
+
+        return new_data
+
+
 class OCIBuildImageSerializer(serializers.Serializer):
     """
     Serializer for building an OCI container image from a Containerfile.
