@@ -5,7 +5,7 @@ import hashlib
 import json
 import requests
 import unittest
-from urllib.parse import urljoin
+from urllib.parse import urljoin, urlparse
 
 from pulp_smash import api, cli, config, exceptions
 from pulp_smash.pulp3.bindings import monitor_task
@@ -17,6 +17,7 @@ from pulp_smash.pulp3.utils import (
 )
 
 from pulp_container.tests.functional.utils import (
+    TOKEN_AUTH_DISABLED,
     core_client,
     gen_container_client,
     gen_container_remote,
@@ -60,6 +61,7 @@ class PullContentTestCase(unittest.TestCase):
         * `Pulp #4460 <https://pulp.plan.io/issues/4460>`_
         """
         cls.cfg = config.get_config()
+        cls.registry_name = urlparse(cls.cfg.get_base_url()).netloc
 
         cls.client = api.Client(cls.cfg, api.code_handler)
         client_api = gen_container_client()
@@ -69,6 +71,8 @@ class PullContentTestCase(unittest.TestCase):
 
         cls.teardown_cleanups = []
 
+        delete_orphans()
+
         with contextlib.ExitStack() as stack:
             # ensure tearDownClass runs if an error occurs here
             stack.callback(cls.tearDownClass)
@@ -127,7 +131,7 @@ class PullContentTestCase(unittest.TestCase):
         2. Call dockerhub API and get blobsums for synced image.
         3. Compare the checksums.
         """
-        # Get local checksums for content synced from remote registy
+        # Get local checksums for content synced from the remote registry
         checksums = [
             content["digest"]
             for content in get_content(self.repo.to_dict())[CONTAINER_CONTENT_NAME]
@@ -145,22 +149,26 @@ class PullContentTestCase(unittest.TestCase):
         image_path = "/v2/{}/manifests/{}".format(self.distribution_with_repo.base_path, "latest")
         latest_image_url = urljoin(self.cfg.get_base_url(), image_path)
 
-        with self.assertRaises(requests.HTTPError) as cm:
-            self.client.get(latest_image_url, headers={"Accept": MEDIA_TYPE.MANIFEST_V1})
+        if TOKEN_AUTH_DISABLED:
+            auth = ()
+        else:
+            with self.assertRaises(requests.HTTPError) as cm:
+                self.client.get(latest_image_url, headers={"Accept": MEDIA_TYPE.MANIFEST_V1})
 
-        content_response = cm.exception.response
-        self.assertEqual(content_response.status_code, 401)
+            content_response = cm.exception.response
+            self.assertEqual(content_response.status_code, 401)
 
-        authenticate_header = content_response.headers["Www-Authenticate"]
-        queries = AuthenticationHeaderQueries(authenticate_header)
-        content_response = requests.get(
-            queries.realm, params={"service": queries.service, "scope": queries.scope}
-        )
-        content_response.raise_for_status()
-        token = content_response.json()["token"]
+            authenticate_header = content_response.headers["Www-Authenticate"]
+            queries = AuthenticationHeaderQueries(authenticate_header)
+            content_response = requests.get(
+                queries.realm, params={"service": queries.service, "scope": queries.scope}
+            )
+            content_response.raise_for_status()
+            token = content_response.json()["token"]
+            auth = BearerTokenAuth(token)
         content_response = requests.get(
             latest_image_url,
-            auth=BearerTokenAuth(token),
+            auth=auth,
             headers={"Accept": MEDIA_TYPE.MANIFEST_V1},
         )
         content_response.raise_for_status()
@@ -188,6 +196,7 @@ class PullContentTestCase(unittest.TestCase):
         """
         registry = cli.RegistryClient(self.cfg)
         registry.raise_if_unsupported(unittest.SkipTest, "Test requires podman/docker")
+        registry.login("-u", "admin", "-p", "password", self.registry_name)
 
         local_url = urljoin(self.cfg.get_base_url(), self.distribution_with_repo.base_path)
 
@@ -211,6 +220,7 @@ class PullContentTestCase(unittest.TestCase):
         """
         registry = cli.RegistryClient(self.cfg)
         registry.raise_if_unsupported(unittest.SkipTest, "Test requires podman/docker")
+        registry.login("-u", "admin", "-p", "password", self.registry_name)
 
         local_url = urljoin(self.cfg.get_base_url(), self.distribution_with_repo_version.base_path)
 
@@ -234,6 +244,7 @@ class PullContentTestCase(unittest.TestCase):
         """
         registry = cli.RegistryClient(self.cfg)
         registry.raise_if_unsupported(unittest.SkipTest, "Test requires podman/docker")
+        registry.login("-u", "admin", "-p", "password", self.registry_name)
 
         local_url = (
             urljoin(self.cfg.get_base_url(), self.distribution_with_repo.base_path)
@@ -258,6 +269,7 @@ class PullContentTestCase(unittest.TestCase):
         """
         registry = cli.RegistryClient(self.cfg)
         registry.raise_if_unsupported(unittest.SkipTest, "Test requires podman/docker")
+        registry.login("-u", "admin", "-p", "password", self.registry_name)
 
         local_url = urljoin(self.cfg.get_base_url(), "inexistentimagename")
         with self.assertRaises(exceptions.CalledProcessError):
@@ -282,6 +294,7 @@ class PullOnDemandContentTestCase(unittest.TestCase):
         * `Pulp #4460 <https://pulp.plan.io/issues/4460>`_
         """
         cls.cfg = config.get_config()
+        cls.registry_name = urlparse(cls.cfg.get_base_url()).netloc
 
         client_api = gen_container_client()
         cls.repositories_api = RepositoriesContainerApi(client_api)
@@ -378,6 +391,7 @@ class PullOnDemandContentTestCase(unittest.TestCase):
         """
         registry = cli.RegistryClient(self.cfg)
         registry.raise_if_unsupported(unittest.SkipTest, "Test requires podman/docker")
+        registry.login("-u", "admin", "-p", "password", self.registry_name)
 
         local_url = urljoin(self.cfg.get_base_url(), self.distribution_with_repo.base_path)
 
@@ -405,6 +419,7 @@ class PullOnDemandContentTestCase(unittest.TestCase):
         """
         registry = cli.RegistryClient(self.cfg)
         registry.raise_if_unsupported(unittest.SkipTest, "Test requires podman/docker")
+        registry.login("-u", "admin", "-p", "password", self.registry_name)
 
         local_url = urljoin(self.cfg.get_base_url(), self.distribution_with_repo_version.base_path)
 
@@ -428,6 +443,7 @@ class PullOnDemandContentTestCase(unittest.TestCase):
         """
         registry = cli.RegistryClient(self.cfg)
         registry.raise_if_unsupported(unittest.SkipTest, "Test requires podman/docker")
+        registry.login("-u", "admin", "-p", "password", self.registry_name)
 
         local_url = (
             urljoin(self.cfg.get_base_url(), self.distribution_with_repo.base_path)
