@@ -2,12 +2,12 @@
 """Tests that recursively add container content to repositories."""
 import unittest
 
-from pulp_smash.pulp3.utils import gen_repo
+from pulp_smash.pulp3.bindings import monitor_task
+from pulp_smash.pulp3.utils import delete_orphans, gen_repo
 
 from pulp_container.tests.functional.utils import (
     gen_container_remote,
     gen_container_client,
-    monitor_task,
 )
 from pulp_container.tests.functional.constants import DOCKERHUB_PULP_FIXTURE_1
 
@@ -509,6 +509,7 @@ class TestRecursiveAdd(unittest.TestCase):
         """Delete things made in setUpClass. addCleanup feature does not work with setupClass."""
         cls.repositories_api.delete(cls.from_repo.pulp_href)
         cls.remotes_api.delete(cls.remote.pulp_href)
+        delete_orphans()
 
     def test_repository_only(self):
         """Passing only a repository does not create a new version."""
